#include "PageLayout.hh"

#include <iostream>
using namespace std;

PageLayout::PageLayout()
  : Glib::ObjectBase(typeid(PageLayout)),
    prop_row_spacing (*this, "row-spacing")
{
  
}

void PageLayout::get_preferred_width_vfunc(int& minimum_width, int& natural_width) const
{
  minimum_width = natural_width = 0;
  
  for(const Gtk::Widget* widget : get_children())
  {
    int widget_minimum_width, widget_natural_width;
    
    widget->get_preferred_width(widget_minimum_width, widget_natural_width);
    
    minimum_width = std::max(minimum_width, widget_minimum_width);
    natural_width = std::max(natural_width, widget_natural_width);
  }
  
  cout << "PageLayout::get_preferred_width_vfunc" << endl;
}

void PageLayout::get_preferred_height_vfunc(int& minimum_height, int& natural_height) const
{
  minimum_height = natural_height = 0;
  int n_vis_children = 0;
  
  for(const Gtk::Widget* widget : get_children())
  {
    if(!widget->get_visible())
      continue;
    
    ++n_vis_children;
    
    int widget_minimum_height, widget_natural_height;
    
    widget->get_preferred_height(widget_minimum_height, widget_natural_height);
    
    minimum_height += widget_minimum_height;
    natural_height += widget_natural_height;
  }
  
  if(n_vis_children > 1)
  {
    int margin = (n_vis_children - 1) * prop_row_spacing.get_value();
    minimum_height += margin;
    natural_height += margin;
  }
  
  cout << "PageLayout::get_preferred_height_vfunc" << endl;
}

void PageLayout::on_size_allocate(Gtk::Allocation& allocation)
{
  cout << "PageLayout::on_size_allocate" << endl;
  
  int x = allocation.get_x(),
      y = allocation.get_y();
  
  int n_expanding = 0,
      min_height = 0;
  
  for(Gtk::Widget* widget : get_children())
  {
    if(!widget->get_visible())
      continue;
    
    int widget_minimum_height, widget_natural_height;
    
    widget->get_preferred_height(widget_minimum_height, widget_natural_height);
    
    min_height += widget_minimum_height;
    
    if(widget->get_vexpand())
      ++n_expanding;
  }
  
  int expanding_height = 0;
  
  if(n_expanding > 0)
    expanding_height = (allocation.get_height() - min_height) / (n_expanding);
  
  for(Gtk::Widget* widget : get_children())
  {
    if(!widget->get_visible())
      continue;
    
    int widget_minimum_width, widget_natural_width;
    
    widget->get_preferred_width(widget_minimum_width, widget_natural_width);
    
    const int available_width = allocation.get_width();
    int width = 0;
    
    if(widget->get_hexpand())
    {
      width = available_width;
    }
    else
    {
      width = widget_minimum_width;
    }
    
    Gtk::Allocation widget_alloc;
    
    widget_alloc.set_width(width);
    widget_alloc.set_x(x + (available_width - width)/2);
    widget_alloc.set_y(y);
    
    int widget_minimum_height, widget_natural_height;
    
    widget->get_preferred_height(widget_minimum_height, widget_natural_height);
    
    if(widget->get_vexpand())
    {
      widget_alloc.set_height(widget_minimum_height);
    }
    else
    {
      widget_alloc.set_height(widget_minimum_height + expanding_height);
    }
    
    widget->set_allocation(widget_alloc);
    
    y += prop_row_spacing.get_value() + widget_alloc.get_height();
  }
}
